
/*
  Copyright (C) 2002-2008  Dmitry V. Levin <ldv@altlinux.org>

  CheckSubst method for packageof.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "chksubst.h"
#include "cwd.h"
#include <cstdio>
#include <cstring>
#include <dirent.h>
#include <sys/stat.h>

int
CheckSubst::add(const char *dn)
{
	CWD     cwd;

	if (chdir(dn) < 0)
		return 0;

	DIR    *dir = opendir(".");

	if (!dir)
		return 0;

	struct dirent *ent;

	while ((ent = readdir(dir)))
	{
		const char *name = ent->d_name;

		if (!name[0] || name[0] == '.')
			continue;

		FILE   *fp = fopen(name, "r");

		if (!fp)
			continue;

		struct stat st;

		if (fstat(fileno(fp), &st))
		{
			fclose(fp);
			continue;
		}

		if (!S_ISREG(st.st_mode))
		{
			fclose(fp);
			continue;
		}

		char    line[PATH_MAX];

		while (fgets(line, sizeof(line), fp))
		{
			for (size_t i = strlen(line) - 1; i > 0; --i)
			{
				if (line[i] == '\n' || line[i] == '\r')
					line[i] = '\0';
				else
					break;
			}

			if (!*line || *line == '#')
				continue;

			string  key(name);
			string  val(line);

			subst[key] = val;
			break;
		}

		fclose(fp);
	}

	closedir(dir);
	return 1;
}

const   string &
CheckSubst::run(const string & str)
{
	const   string & val = subst[str];

	return val.length()? val : str;
}
