
/*
  Copyright (C) 2002-2008  Dmitry V. Levin <ldv@altlinux.org>

  list packages owning files.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "query.h"
#include <cstdlib>
#include <cerrno>
#include <getopt.h>

int     verbose;

static void
usage(FILE * stream, int status)
{
	fprintf(stream,
		"%s - list packages owning files.\n"
		"Usage: %s [-r path | --root=path] [-s path | --substitute=path]\n"
		"       [-i path | --ignore=path] [-f file | --from=file]\n"
		"       [-v | --verbose] [--help] [files...]\n",
		program_invocation_short_name, program_invocation_short_name);
	fputs("\nReport bugs to http://bugs.altlinux.ru/\n", stream);
	exit(status);
}

static struct option const longopts[] = {
	{"substitute", required_argument, 0, 's'},
	{"ignore", required_argument, 0, 'i'},
	{"from", required_argument, 0, 'f'},
	{"root", required_argument, 0, 'r'},
	{"verbose", no_argument, 0, 'v'},
	{"help", no_argument, 0, 'h'},
	{0, no_argument, 0, 0}
};

int
main(int ac, char *const *av)
{
	if (ac < 1)
		usage(stderr, EXIT_FAILURE);

	int     optc;
	FILE   *input = 0;
	const char *prefix = 0;
	CheckIgnore chkign;
	CheckSubst chksubst;

	while ((optc =
		getopt_long(ac, av, "s:i:f:r:vh", longopts, (int *) 0)) != -1)
		switch (optc)
		{
			case 'r':
				prefix = optarg;
				break;

			case 's':
				if (!chksubst.add(optarg))
					error(EXIT_FAILURE, errno,
					      "subst: %s", optarg);
				break;

			case 'i':
				if (!chkign.add(optarg))
					error(EXIT_FAILURE, errno,
					      "ignore: %s", optarg);
				break;

			case 'f':
				if (input)
					usage(stderr, EXIT_FAILURE);

				if (!(input = fopen(optarg, "r")))
					error(EXIT_FAILURE, errno,
					      "open: %s", optarg);

				break;

			case 'v':
				++verbose;
#if 0
				rpmIncreaseVerbosity();
#endif
				break;

			case 'h':
				usage(stdout, EXIT_SUCCESS);

			default:
				usage(stderr, EXIT_FAILURE);
		}

	chkign.compile();

	Query   query(chkign, chksubst, prefix);
	int     rc = EXIT_SUCCESS;

	if (input)
	{
		if (optind != ac)
			usage(stderr, EXIT_FAILURE);

		if (query.input(input) != EXIT_SUCCESS)
			rc = EXIT_FAILURE;
	} else
	{
		if (optind >= ac)
			usage(stderr, EXIT_FAILURE);

		if (query.args(ac - optind, av + optind))
			rc = EXIT_FAILURE;
	}

	query.dump();

	return verbose ? rc : EXIT_SUCCESS;
}
