
/*
  Copyright (C) 2002-2006  Dmitry V. Levin <ldv@altlinux.org>

  query for packages owning files.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "query.h"
#include <cstdlib>
#include <cstring>
#include <climits>

int
Query::filename(const char *fname)
{
	PkgMapType inset, outset;
	int     rc = query->run(inset, fname);

	PkgMapType::const_iterator i;
	for (i = inset.begin(); i != inset.end(); i++)
	{
		const string out = chksubst.run(i->first);

#ifdef DEBUG
		error(0, 0, "in=%s, out=%s", i->first.c_str(), out.c_str());
#endif
		if (chkign.is_ignore(out))
			break;
		outset[out] = 1;
	}

	if (i == inset.end())
		for (i = outset.begin(); i != outset.end(); i++)
			packages[i->first] = 1;

	return rc;
}

int
Query::input(FILE * fp)
{
	int     rc = EXIT_SUCCESS;
	char    line[PATH_MAX];

	while (fgets(line, sizeof(line), fp))
	{
		for (size_t i = strlen(line) - 1; i > 0; --i)
		{
			if (line[i] == '\n' || line[i] == '\r')
				line[i] = '\0';
			else
				break;
		}

		if (!line[0])
			continue;

		if (filename(line) != EXIT_SUCCESS)
			rc = EXIT_FAILURE;
	}

	if (!feof(fp))
		error(EXIT_SUCCESS, errno, "input");

	fclose(fp);

	return rc;
}

int
Query::args(int ac, const char *const *av)
{
	int     rc = EXIT_SUCCESS;

	for (int i = 0; i < ac; ++i)
		if (filename(av[i]) != EXIT_SUCCESS)
			rc = EXIT_FAILURE;

	return rc;
}

void
Query::dump()
{
	for (PkgMapType::const_iterator i = packages.begin();
	     i != packages.end(); i++)
		puts(i->first.c_str());
}
